# encoding: utf-8

__author__ = u'Yonka'

# /** terminal=type **/
TERMINAL_TRADE = 0
TERMINAL_QUOTE = 1

# /*** =terminal=<>=center=proto ***/
HEARTBEAT = 0
ACK = 1
MANAGERLOGIN = 2  # terminal=login=to=center
CONNECTFRONT = 3  # quant=center=send=order
TERMINALLOGIN = 4
STRATEGYLOGIN = 5

# /******* =data=type= *******/
STRATEGY = 0

CTP_FUTURE = 1
ZX_OPTION = 2
ZX_STOCK = 3


# /*************************
# * strategy functioncode
# * *************************/
# /**request**/
REQ_ST_POTITIONDETAIL = 10000
REQ_ST_ACCOUNTETAIL = 10001
REQ_ST_POTITIONDSTATICS = 10002
REQ_ST_ORDER = 10003
REQ_ST_CANCELORDER = 10004
REQ_ST_SUBSCRIBE = 10005
REQ_ST_UNSUBSCRIBE = 10006
REQ_ST_ORDERLIST = 10007
REQ_ST_DATA = 10008

# /**response**/
RESP_ST_POTITIONDETAIL = 1100
RESP_ST_ACCOUNTETAIL = 11001
RESP_ST_POTITIONDSTATICS = 11002
RESP_ST_ORDER = 11003
RESP_ST_CANCELORDER = 11004
NOTIFY_ST_ORDERERROR = 11005
NOTIFY_ST_ORDERDETAIL = 11006
NOTIFY_ST_SUBSCRIBE = 11007
NOTIFY_ST_REQERROR = 11008
NOTITY_ST_CTPORDERID = 11009
NOTIFY_ST_UNSUBSCRIBE = 11010
RESP_ST_LOGIN = 11011
RESP_ST_SUBSCRIBE = 11012
RESP_ST_UNSUBSCRIBE = 11013
RESP_ST_ERROR = 11014

# request
REQ_ACCOUNTDETAIL = 100
REQ_POSITIONDETAIL = 101
REQ_POSITIONSTATICS = 102
REQ_ORDER = 103
REQ_CACELORDER = 104
REQ_SUBSCRIBE = 105
REQ_UNSUBSCRIBE = 106

REQ_QUOTEDATA = 107
REQ_QRYTRADE = 108
REQ_QRYORDER = 109
REQ_INSTRUMENTS = 110
REQ_QRYINSTRUMENT_COMMISSIONRATEFIELD = 111
REQ_QRYINSTRUMENT_MARGINRATEFIELD = 112

# response
RESP_ACCOUNTDETAIL = 500
RESP_POSITIONDETAIL = 501
RESP_POSITIONSTATICS = 502
RESP_ERROR = 503
RESP_ORDER = 504
RESP_CANCELORDER = 505
RESP_SUBSCRIBE = 506
RESP_UNSUBSCRIBE = 507
RESP_QUOTEDATA = 508
RESP_FINISHED = 509
RESP_QRYTRADE = 510
RESP_QRYORDER = 512
RESP_INSTRUMENTS = 513
RESP_QRYINSTRUMENT_COMMISSIONRATEFIELD = 514
RESP_QRYINSTRUMENT_MARGINRATEFIELD = 515

# notify
NOTIFY_ORDERDETAIL = 600
NOTIFY_ORDERERROR = 601
NOTIFY_TRADEDETAIL = 602
NOTIFY_CANCELERROR = 603
NOTIFY_LINKERROR = 604

# /*----------------------------------------------------------------*/
# /******************** =1001~2000=zx=option ************************/
#  行情
REQ_OPT_PRICE_QRY = 1001  # 期权行情查询public final
#  static int public
#  final static int 395
#  交易
REQ_OPTCODE_QRY = 1002  # 期权代码信息查询=public final
#  static int 338000
REQ_OPTCODE_IQRY = 1003  # 获取期权标的证券信息=public
#  final static int
#  338001
REQ_OPTCODE_ENTER = 1004  # 期权代码输入确认public final
#  static int 338002
REQ_OPT_ENTRADE = 1005  # 期权可交易数量获取public final
#  static int 338010
REQ_OPT_ENTRUST = 1006  # 期权委托public final static
#  int public final
#  static int 338011
REQ_OPT_WITHDRAW = 1007  # 期权撤单public final
#  static int public
#  final static int
#  338012
REQ_OPT_ENTRUSTQRY = 1008  # 期权委托查询public final
#  static int public
#  final static int
#  338020
REQ_OPT_REALTIME_QRY = 1009  # 期权成交查询public final
#  static int
#  public final
#  static int 338021
REQ_OPT_ASSET_QRY = 1010  # 期权资产查询public final
#  static int public
#  final static int
#  338022
REQ_OPT_HOLD_QRY = 1011  # 期权持仓查询public final
#  static int public
#  final static int
#  338023
REQ_OPT_EXEASSIGN_QRY = 1012  # 期权行权指派查询public
#  final static int
#  338024
REQ_ASSET_OPTEXEDELIVER_QRY = 1013  # 期权行权交割信息查询public
#  final
#  static
#  int
#  338025

REQ_ASSET_DEBTINFO_QRY = 1014  # 期权行权指派欠资欠券查询
#  338026
REQ_OPT_UNDERLYAMOUNT_PROMPT = 1015  # 期权标的证券数量提示public
#  final
#  static
#  int
#  338027
REQ_OPT_OPTHOLDREAL_PROMPT = 1016  # 客户期权持仓合约信息提示
#  338028
REQ_ASSET_ASSETCOVERSTOCK_QRY = 1017  # =客户备兑证券不足查询=
#  338029

#  历史交易查询
REQ_HIS_OPTENTRUST_QRY = 1050  # 历史期权委托查询public
#  final static int
#  339800
REQ_HIS_OPTBUSINESS_QRY = 1051  # 历史期权成交查询public
#  final static
#  int 339801
REQ_HIS_OPTDELIVER_QRY = 1052  # 历史期权交割信息查询public
#  final static int
#  339803
REQ_HIS_EXEASSIGN_QRY = 1053  # 历史期权行权指派查询public
#  final static int
#  339804
REQ_HIS_OPTEXEDELIVER_QRY = 1054  # 历史期权行权交割信息查询
#  339805
REQ_HIS_OPTSTATEMENT_QRY = 1055  # 历史期权对账单查询public
#  final static
#  int 339806

#  resp
RESP_OPT_ENTRUST = 1501  # 下单返回
RESP_OPT_ENTRUST_QRY = 1502  # 期权委托查询返回
RESP_OPT_REALTIME_QRY = 1503  # 期权成交查询返回
RESP_OPT_ASSET_QRY = 1504  # 期权资产查询返回
RESP_OPT_HOLD_QRY = 1505  # 持仓查询返回
RESP_OPT_WITHDRAW = 1506  # 撤单返回
RESP_OPT_ENTRADE = 1507  # 获取可交易数量返回
RESP_OPTCODE_ENTER = 1508  # 期权代码输入确认返回

NOTIFY_REALTIME = 1600  # 成交notify


# /**********以下为期货属性常量值**************/
#  下单类型
PRTP_FIX = 11  # 限价单
PRTP_MARKET = 12  # 市价单

#  操作指令常量
OPT_OPEN_LONG = 0
OPT_CLOSE_LONG_HISTORY = 1
OPT_CLOSE_LONG_TODAY = 2
OPT_OPEN_SHORT = 3
OPT_CLOSE_SHORT_HISTORY = 4
OPT_CLOSE_SHORT_TODAY = 5
OPT_CLOSE_LONG_TODAY_FIRST = 6
OPT_CLOSE_LONG_HISTORY_FIRST = 7
OPT_CLOSE_SHORT_TODAY_FIRST = 8
OPT_CLOSE_SHORT_HISTORY_FIRST = 9
OPT_CLOSE_LONG_TODAY_HISTORY_THEN_OPEN_SHORT = 10
OPT_CLOSE_LONG_HISTORY_TODAY_THEN_OPEN_SHORT = 11
OPT_CLOSE_SHORT_TODAY_HISTORY_THEN_OPEN_LONG = 12
OPT_CLOSE_SHORT_HISTORY_TODAY_THEN_OPEN_LONG = 13
OPT_CLOSE_LONG = 14
OPT_CLOSE_SHORT = 15
OPT_OPEN = 16
OPT_CLOSE = 17
OPT_BUY = 18
OPT_SELL = 19
OPT_FIN_BUY = 20
OPT_SLO_SELL = 21
OPT_BUY_SECU_REPAY = 22
OPT_DIRECT_SECU_REPAY = 23
OPT_SELL_CASH_REPAY = 24
OPT_DIRECT_CASH_REPAY = 25

# /************************期权相并常量值************************/
# entrust_prop	委托属性（数据字典项1200)
ENTRUST_PROP_GFD = u"0"  # 限价GFD
ENTRUST_PROP_OPA = u"OPA"  # 限价即时全部成交否则撤单
ENTRUST_PROP_OPB = u"OPB"  # 市价即时成交剩余撤单
ENTRUST_PROP_OPC = u"OPC"  # 市价即时全部成交否则撤单
ENTRUST_PROP_OPD = u"OPD"  # 市价剩余转限价
ENTRUST_PROP_OTE = u"OTE"  # 期权行权
ENTRUST_PROP_OTU = u"OTU"  # 备兑证券划转
# option_type	期权类型（数据字典项36002)
OPTION_TYPE_C = u"C"  # 认购
OPTION_TYPE_P = u"P"  # 认沽
# entrust_oc	开平仓方向（数据字典项36300)
ENTRUST_OC_O = u"O"  # 开仓
ENTRUST_OC_C = u"C"  # 平仓
ENTRUST_OC_X = u"X"  # 行权
ENTRUST_OC_A = u"A"  # 自动行权
# covered_flag	备兑标志（数据字典项36301)
COVERED_FLAG_1 = u"1"  # 备兑
# OPTHOLD_TYPE	期权持仓类别（数据字典项36013）
OPTHOLD_TYPE_0 = u"0"  # 权利方
OPTHOLD_TYPE_1 = u"1"  # 义务方
OPTHOLD_TYPE_2 = u"2"  # 备兑方
# OPT_OPEN_STATUS	期权开仓限制（数据字典项36005）
OPT_OPEN_STATUS_0 = u"0"  # 否
OPT_OPEN_STATUS_1 = u"1"  # 是
# OPTION_FLAG	期权合约挂牌标志（数据字典项36008）
OPTION_FLAG_A = u"A"  # 当日新挂牌合约
OPTION_FLAG_E = u"E"  # 存续合约
OPTION_FLAG_D = u"D"  # 当日摘牌合约
# OPT_FINAL_STATUS	期权开仓限制（数据字典项36006）
OPT_FINAL_STATUS_0 = u"0"  # 否
OPT_FINAL_STATUS_1 = u"1"  # 是
# FINANCE_TYPE	金融品种（数据字典项1310）
FINANCE_TYPE_8 = u"8"  # 期权
# BUSINESS_STATUS	成交状态
BUSINESS_STATUS_0 = u"0"  # 成交
BUSINESS_STATUS_2 = u"2"  # 废单
BUSINESS_STATUS_4 = u"4"  # 确认
# OPTEXE_STG_KIND	自动行权策略类别（数据字典36035）
OPTEXE_STG_KIND_0 = u"0"  # 实值X元即行权
OPTEXE_STG_KIND_1 = u"1"  # 盈利X元即行权
OPTEXE_STG_KIND_2 = u"2"  # 盈利百分比即行权
OPTEXE_STG_KIND_3 = u"3"  # 亏损百分比即行权
OPTEXE_STG_KIND_4 = u"4"  # 实值百分比即行权
# ENTRUST_SRC	委托来源（数据字典36017）
ENTRUST_SRC_0 = u"0"  # 个人投资者发起
ENTRUST_SRC_1 = u"1"  # 交易所发起
ENTRUST_SRC_2 = u"2"  # 会员发起
ENTRUST_SRC_3 = u"3"  # 机构投资者发起
ENTRUST_SRC_4 = u"4"  # 自营交易发起
ENTRUST_SRC_5 = u"5"  # 流动性服务提供商发起
# OPTRIKE_TYPE	风险类别（数据字典36031）
OPTRIKE_TYPE_1 = u"1"  # 关注
OPTRIKE_TYPE_2 = u"2"  # 警告
OPTRIKE_TYPE_3 = u"3"  # 强平
# DEBT_TYPE	负债类型（数据字典36037）
DEBT_TYPE_1 = u"1"  # 欠资
DEBT_TYPE_2 = u"2"  # 欠券



# encoding: utf-8

__author__ = u'Yonka'


def can_not_init(self, v):
    raise AssertionError(u"can not initiate")


class StrategyErrorCode(object):
    _instances = {}

    def __init__(self, v):
        self.v = v
        self.__class__._instances[v] = self

    def get_value(self):
        return self.v

    def from_value(self, v):
        return self.__class__._instances.get(v)

    def __str__(self):
        return u"StrategyErrorCode(%d)" % self.v

    @classmethod
    def is_success(cls, code):
        return code == 0


StrategyErrorCode.VALIDATE_SUCCESS = StrategyErrorCode(0)
StrategyErrorCode.VALIDATE_STRATEGY_NOTEXISTS = StrategyErrorCode(10001)
StrategyErrorCode.VALIDATE_ACCOUNT_NOTEXISTS = StrategyErrorCode(10002)
StrategyErrorCode.VALIDATE_ACCOUNT_WRONG_PASSWORD = StrategyErrorCode(10002)
StrategyErrorCode.SERVER_ERROR5000 = StrategyErrorCode(5000)
StrategyErrorCode.ORDER_MSG_CODE11000 = StrategyErrorCode(11000)
StrategyErrorCode.ORDER_MSG_CODE11001 = StrategyErrorCode(11001)
StrategyErrorCode.ORDER_MSG_CODE11002 = StrategyErrorCode(11002)
StrategyErrorCode.ORDER_MSG_CODE11003 = StrategyErrorCode(11003)
StrategyErrorCode.ORDER_MSG_CODE11004 = StrategyErrorCode(11004)

StrategyErrorCode.__init__ = can_not_init



# encoding: utf-8

from __future__ import absolute_import
from abc import ABCMeta, abstractmethod

__author__ = u'Yonka'


class AbstractProtocolMessage(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def to_stream_bytes(self):
        pass



# encoding: utf-8
from __future__ import absolute_import
import unittest

from sd.utils.enum import EnumMetaCls, CommonNameValueEnum, can_not_init

__author__ = u'yonka'


class EntrustBS(CommonNameValueEnum):
    __metaclass__ = EnumMetaCls

    def __str__(self):
        return u"EntrustBS(%d)" % self.v


EntrustBS.ENTRUST_BUY = EntrustBS(48)
EntrustBS.ENTRUST_SELL = EntrustBS(49)

EntrustBS.__init__ = can_not_init


class HedgeFlagType(CommonNameValueEnum):
    __metaclass__ = EnumMetaCls

    def __str__(self):
        return u"HedgeFlagType(%d)" % self.v


HedgeFlagType.HEDGE_FLAG_SPECULATION = HedgeFlagType(49)
HedgeFlagType.HEDGE_FLAG_ARBITRAGE = HedgeFlagType(50)
HedgeFlagType.HEDGE_FLAG_HEDGE = HedgeFlagType(51)

HedgeFlagType.__init__ = can_not_init


class OffsetFlagType(CommonNameValueEnum):
    __metaclass__ = EnumMetaCls

    def __str__(self):
        return u"OffsetFlagType(%d)" % self.v


OffsetFlagType.EOFF_THOST_FTDC_OF_INVALID = OffsetFlagType(-1)
OffsetFlagType.EOFF_THOST_FTDC_OF_Open = OffsetFlagType(48)
OffsetFlagType.EOFF_THOST_FTDC_OF_Close = OffsetFlagType(49)
OffsetFlagType.EOFF_THOST_FTDC_OF_ForceClose = OffsetFlagType(50)
OffsetFlagType.EOFF_THOST_FTDC_OF_CloseToday = OffsetFlagType(51)
OffsetFlagType.EOFF_THOST_FTDC_OF_CloseYesterday = OffsetFlagType(52)
OffsetFlagType.EOFF_THOST_FTDC_OF_ForceOff = OffsetFlagType(53)
OffsetFlagType.EOFF_THOST_FTDC_OF_LocalForceClose = OffsetFlagType(54)

OffsetFlagType.__init__ = can_not_init


class EntrustSubmitStatus(CommonNameValueEnum):
    __metaclass__ = EnumMetaCls

    def __str__(self):
        return u"EntrustSubmitStatus(%d)" % self.v


EntrustSubmitStatus.ENTRUST_SUBMIT_STATUS_InsertSubmitted = EntrustSubmitStatus(48)
EntrustSubmitStatus.ENTRUST_SUBMIT_STATUS_CancelSubmitted = EntrustSubmitStatus(49)
EntrustSubmitStatus.ENTRUST_SUBMIT_STATUS_ModifySubmitted = EntrustSubmitStatus(50)
EntrustSubmitStatus.ENTRUST_SUBMIT_STATUS_OSS_Accepted = EntrustSubmitStatus(51)
EntrustSubmitStatus.ENTRUST_SUBMIT_STATUS_InsertRejected = EntrustSubmitStatus(52)
EntrustSubmitStatus.ENTRUST_SUBMIT_STATUS_CancelRejected = EntrustSubmitStatus(53)
EntrustSubmitStatus.ENTRUST_SUBMIT_STATUS_ModifyRejected = EntrustSubmitStatus(54)

EntrustSubmitStatus.__init__ = can_not_init


class EntrustStatus(CommonNameValueEnum):
    __metaclass__ = EnumMetaCls

    def __str__(self):
        return u"EntrustSubmitStatus(%d)" % self.v


EntrustStatus.ENTRUST_STATUS_CREATED = EntrustStatus(10)
EntrustStatus.ENTRUST_STATUS_UNREPORTED = EntrustStatus(13)
EntrustStatus.ENTRUST_STATUS_REPORTED = EntrustStatus(15)
EntrustStatus.ENTRUST_STATUS_NOT_TRADE = EntrustStatus(16)
EntrustStatus.ENTRUST_STATUS_PART_SUCC = EntrustStatus(17)
EntrustStatus.ENTRUST_STATUS_SUCCEEDED = EntrustStatus(19)
EntrustStatus.ENTRUST_STATUS_UNREPORTED_CANCEL = EntrustStatus(20)
EntrustStatus.ENTRUST_STATUS_REPORTED_CANCEL = EntrustStatus(23)
EntrustStatus.ENTRUST_STATUS_CANCELED = EntrustStatus(54)
EntrustStatus.ENTRUST_STATUS_NOT_FUND = EntrustStatus(100)

EntrustStatus.__init__ = can_not_init


class TestEnum(unittest.TestCase):
    def test_init_entrust_bs(self):
        print EntrustBS.ENTRUST_BUY
        res = False
        try:
            EntrustBS(48)
        except AssertionError:
            res = True
        assert res

    def test_init_hedge_flag_type(self):
        print HedgeFlagType.HEDGE_FLAG_SPECULATION
        res = False
        try:
            HedgeFlagType(49)
        except AssertionError:
            res = True
        assert res

    def test_init_offset_flag_type(self):
        print OffsetFlagType.EOFF_THOST_FTDC_OF_INVALID
        res = False
        try:
            OffsetFlagType(49)
        except AssertionError:
            res = True
        assert res

    def test_init_entrust_submit_status(self):
        print EntrustSubmitStatus.ENTRUST_SUBMIT_STATUS_InsertSubmitted
        res = False
        try:
            EntrustSubmitStatus(49)
        except AssertionError:
            res = True
        assert res

    def test_init_entrust_status(self):
        print EntrustStatus.ENTRUST_STATUS_CREATED
        res = False
        try:
            EntrustStatus(100)
        except AssertionError:
            res = True
        assert res



# encoding: utf-8
from __future__ import with_statement
from __future__ import absolute_import
import logging
import threading
from sd.communication.client.connection.interface import AbstractBaseConn

from sd.communication.client.protocol.protocol import LengthFramer
from sd.utils.io.io import SocketIO

__author__ = u'Yonka'


class BaseConn(AbstractBaseConn):
    def __init__(
            self,
            s
    ):
        self.s = s
        # isOpen rather than open to avoid conflicting with key word open
        self.isOpen = True
        input_stream = SocketIO(s, u"rw")  # python does not separate input and output interface
        self.framer = LengthFramer(input_stream)
        self._lock = threading.Lock()

    def close(self):
        with self._lock:
            if self.s is not None:
                try:
                    self.s.close()
                except Exception, e:
                    logging.error(u"close, exception: %s", e)
                self.s = None
            self.isOpen = False

    def write(self, protocol_message):
        result = False
        if not self.isOpen:
            return result
        try:
            self.framer.frame_msg(protocol_message)
            result = True
        except Exception, e:
            logging.error(u"Write message error: %s", e, exc_info=True)
            self.close()
        return result

    def read(self):
        result = None
        if not self.isOpen:
            return result
        try:
            result = self.framer.next_msg()
        except Exception, e:
            logging.error(u"Read message error: %s", e, exc_info=True)
            self.close()
        return result

    def is_open(self):
        return self.isOpen



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: quotedatastructs.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)




DESCRIPTOR = _descriptor.FileDescriptor(
    name='quotedatastructs.proto',
    package='proto',
    serialized_pb='\n\x16quotedatastructs.proto\x12\x05proto\"/\n\rQueueDataBean\x12\x10\n\x08\x64\x61taType\x18\x01 \x02(\t\x12\x0c\n\x04\x64\x61ta\x18\x02 \x02(\x0c\"&\n\x0fHistoryDataBean\x12\x13\n\x0bhistoryData\x18\x01 \x03(\x0c\"\xf5\x04\n\x08TickBean\x12\x14\n\x0cinstrumentID\x18\x01 \x02(\t\x12\x12\n\ntradingDay\x18\x02 \x02(\x05\x12\x11\n\tlastPrice\x18\x03 \x02(\x02\x12\x15\n\rpreClosePrice\x18\x04 \x02(\x02\x12\x11\n\topenPrice\x18\x05 \x02(\x02\x12\x14\n\x0chighestPrice\x18\x06 \x02(\x02\x12\x13\n\x0blowestPrice\x18\x07 \x02(\x02\x12\x0e\n\x06volume\x18\x08 \x01(\x03\x12\x10\n\x08turnover\x18\t \x01(\x03\x12\x14\n\x0copenInterest\x18\n \x01(\x03\x12\x17\n\x0fupperLimitPrice\x18\x0b \x02(\x02\x12\x17\n\x0flowerLimitPrice\x18\x0c \x02(\x02\x12\x12\n\nupdateTime\x18\r \x02(\x05\x12\x12\n\ncontractID\x18\x0e \x01(\t\x12\x1c\n\x14underlyingSecurityID\x18\x0f \x01(\t\x12\x11\n\tcallOrPut\x18\x10 \x01(\t\x12\x14\n\x0c\x65xerciseDate\x18\x11 \x01(\x05\x12\x10\n\x08preDelta\x18\x12 \x01(\x03\x12\x10\n\x08\x63urrDela\x18\x13 \x01(\x03\x12\x11\n\tnumTrades\x18\x14 \x01(\x03\x12\x16\n\x0etotalBidVolume\x18\x15 \x01(\x03\x12\x16\n\x0etotalAskVolume\x18\x16 \x01(\x03\x12\x1b\n\x13weightedAvgBidPrice\x18\x17 \x01(\x02\x12\x1b\n\x13weightedAvgAskPrice\x18\x18 \x01(\x02\x12\x10\n\x08\x61skPrice\x18\x19 \x03(\x02\x12\x11\n\taskVolume\x18\x1a \x03(\x05\x12\x10\n\x08\x62idPrice\x18\x1b \x03(\x02\x12\x11\n\tbidVolume\x18\x1c \x03(\x05\x12\x13\n\x0bsettlePrice\x18\x1d \x01(\x02\"\xf1\x01\n\tKLineBean\x12\x14\n\x0cinstrumentID\x18\x01 \x02(\t\x12\x12\n\ntradingDay\x18\x02 \x02(\x05\x12\x12\n\nupdateTime\x18\x03 \x02(\x05\x12\x11\n\topenPrice\x18\x04 \x02(\x02\x12\x11\n\thighPrice\x18\x05 \x02(\x02\x12\x10\n\x08lowPrice\x18\x06 \x02(\x02\x12\x12\n\nclosePrice\x18\x07 \x02(\x02\x12\x0e\n\x06volume\x18\x08 \x02(\x03\x12\x0f\n\x07turover\x18\t \x02(\x03\x12\x12\n\nmatchItems\x18\n \x02(\x03\x12\x10\n\x08interest\x18\x0b \x02(\x05\x12\x13\n\x0bsettlePrice\x18\x0c \x01(\x02\x42\'\n\x15\x63om.quant.quote.protoB\x0eQuoteDataProto')

_QUEUEDATABEAN = _descriptor.Descriptor(
    name='QueueDataBean',
    full_name='proto.QueueDataBean',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='dataType', full_name='proto.QueueDataBean.dataType', index=0,
            number=1, type=9, cpp_type=9, label=2,
            has_default_value=False, default_value=unicode("", "utf-8"),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='data', full_name='proto.QueueDataBean.data', index=1,
            number=2, type=12, cpp_type=9, label=2,
            has_default_value=False, default_value="",
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    options=None,
    is_extendable=False,
    extension_ranges=[],
    serialized_start=33,
    serialized_end=80,
)

_HISTORYDATABEAN = _descriptor.Descriptor(
    name='HistoryDataBean',
    full_name='proto.HistoryDataBean',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='historyData', full_name='proto.HistoryDataBean.historyData', index=0,
            number=1, type=12, cpp_type=9, label=3,
            has_default_value=False, default_value=[],
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    options=None,
    is_extendable=False,
    extension_ranges=[],
    serialized_start=82,
    serialized_end=120,
)

_TICKBEAN = _descriptor.Descriptor(
    name='TickBean',
    full_name='proto.TickBean',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='instrumentID', full_name='proto.TickBean.instrumentID', index=0,
            number=1, type=9, cpp_type=9, label=2,
            has_default_value=False, default_value=unicode("", "utf-8"),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='tradingDay', full_name='proto.TickBean.tradingDay', index=1,
            number=2, type=5, cpp_type=1, label=2,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='lastPrice', full_name='proto.TickBean.lastPrice', index=2,
            number=3, type=2, cpp_type=6, label=2,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='preClosePrice', full_name='proto.TickBean.preClosePrice', index=3,
            number=4, type=2, cpp_type=6, label=2,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='openPrice', full_name='proto.TickBean.openPrice', index=4,
            number=5, type=2, cpp_type=6, label=2,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='highestPrice', full_name='proto.TickBean.highestPrice', index=5,
            number=6, type=2, cpp_type=6, label=2,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='lowestPrice', full_name='proto.TickBean.lowestPrice', index=6,
            number=7, type=2, cpp_type=6, label=2,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='volume', full_name='proto.TickBean.volume', index=7,
            number=8, type=3, cpp_type=2, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='turnover', full_name='proto.TickBean.turnover', index=8,
            number=9, type=3, cpp_type=2, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='openInterest', full_name='proto.TickBean.openInterest', index=9,
            number=10, type=3, cpp_type=2, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='upperLimitPrice', full_name='proto.TickBean.upperLimitPrice', index=10,
            number=11, type=2, cpp_type=6, label=2,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='lowerLimitPrice', full_name='proto.TickBean.lowerLimitPrice', index=11,
            number=12, type=2, cpp_type=6, label=2,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='updateTime', full_name='proto.TickBean.updateTime', index=12,
            number=13, type=5, cpp_type=1, label=2,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='contractID', full_name='proto.TickBean.contractID', index=13,
            number=14, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=unicode("", "utf-8"),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='underlyingSecurityID', full_name='proto.TickBean.underlyingSecurityID', index=14,
            number=15, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=unicode("", "utf-8"),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='callOrPut', full_name='proto.TickBean.callOrPut', index=15,
            number=16, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=unicode("", "utf-8"),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='exerciseDate', full_name='proto.TickBean.exerciseDate', index=16,
            number=17, type=5, cpp_type=1, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='preDelta', full_name='proto.TickBean.preDelta', index=17,
            number=18, type=3, cpp_type=2, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='currDela', full_name='proto.TickBean.currDela', index=18,
            number=19, type=3, cpp_type=2, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='numTrades', full_name='proto.TickBean.numTrades', index=19,
            number=20, type=3, cpp_type=2, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='totalBidVolume', full_name='proto.TickBean.totalBidVolume', index=20,
            number=21, type=3, cpp_type=2, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='totalAskVolume', full_name='proto.TickBean.totalAskVolume', index=21,
            number=22, type=3, cpp_type=2, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='weightedAvgBidPrice', full_name='proto.TickBean.weightedAvgBidPrice', index=22,
            number=23, type=2, cpp_type=6, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='weightedAvgAskPrice', full_name='proto.TickBean.weightedAvgAskPrice', index=23,
            number=24, type=2, cpp_type=6, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='askPrice', full_name='proto.TickBean.askPrice', index=24,
            number=25, type=2, cpp_type=6, label=3,
            has_default_value=False, default_value=[],
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='askVolume', full_name='proto.TickBean.askVolume', index=25,
            number=26, type=5, cpp_type=1, label=3,
            has_default_value=False, default_value=[],
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='bidPrice', full_name='proto.TickBean.bidPrice', index=26,
            number=27, type=2, cpp_type=6, label=3,
            has_default_value=False, default_value=[],
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='bidVolume', full_name='proto.TickBean.bidVolume', index=27,
            number=28, type=5, cpp_type=1, label=3,
            has_default_value=False, default_value=[],
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='settlePrice', full_name='proto.TickBean.settlePrice', index=28,
            number=29, type=2, cpp_type=6, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    options=None,
    is_extendable=False,
    extension_ranges=[],
    serialized_start=123,
    serialized_end=752,
)

_KLINEBEAN = _descriptor.Descriptor(
    name='KLineBean',
    full_name='proto.KLineBean',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='instrumentID', full_name='proto.KLineBean.instrumentID', index=0,
            number=1, type=9, cpp_type=9, label=2,
            has_default_value=False, default_value=unicode("", "utf-8"),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='tradingDay', full_name='proto.KLineBean.tradingDay', index=1,
            number=2, type=5, cpp_type=1, label=2,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='updateTime', full_name='proto.KLineBean.updateTime', index=2,
            number=3, type=5, cpp_type=1, label=2,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='openPrice', full_name='proto.KLineBean.openPrice', index=3,
            number=4, type=2, cpp_type=6, label=2,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='highPrice', full_name='proto.KLineBean.highPrice', index=4,
            number=5, type=2, cpp_type=6, label=2,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='lowPrice', full_name='proto.KLineBean.lowPrice', index=5,
            number=6, type=2, cpp_type=6, label=2,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='closePrice', full_name='proto.KLineBean.closePrice', index=6,
            number=7, type=2, cpp_type=6, label=2,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='volume', full_name='proto.KLineBean.volume', index=7,
            number=8, type=3, cpp_type=2, label=2,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='turover', full_name='proto.KLineBean.turover', index=8,
            number=9, type=3, cpp_type=2, label=2,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='matchItems', full_name='proto.KLineBean.matchItems', index=9,
            number=10, type=3, cpp_type=2, label=2,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='interest', full_name='proto.KLineBean.interest', index=10,
            number=11, type=5, cpp_type=1, label=2,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='settlePrice', full_name='proto.KLineBean.settlePrice', index=11,
            number=12, type=2, cpp_type=6, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    options=None,
    is_extendable=False,
    extension_ranges=[],
    serialized_start=755,
    serialized_end=996,
)

DESCRIPTOR.message_types_by_name['QueueDataBean'] = _QUEUEDATABEAN
DESCRIPTOR.message_types_by_name['HistoryDataBean'] = _HISTORYDATABEAN
DESCRIPTOR.message_types_by_name['TickBean'] = _TICKBEAN
DESCRIPTOR.message_types_by_name['KLineBean'] = _KLINEBEAN


class QueueDataBean(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _QUEUEDATABEAN

    # @@protoc_insertion_point(class_scope:proto.QueueDataBean)


class HistoryDataBean(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _HISTORYDATABEAN

    # @@protoc_insertion_point(class_scope:proto.HistoryDataBean)


class TickBean(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _TICKBEAN

    # @@protoc_insertion_point(class_scope:proto.TickBean)


class KLineBean(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _KLINEBEAN

    # @@protoc_insertion_point(class_scope:proto.KLineBean)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(),
                                                '\n\025com.quant.quote.protoB\016QuoteDataProto')
# @@protoc_insertion_point(module_scope)



# -*- coding:utf-8 -*-
from sd.agent.agent_type import AgentType
from sd.configuration import logconfig
from sd.strategy.bean.bean import StrategyConnectionInfo, StrategyConfiguration

__author__ = 'patronfeng'


############只需要修改服务器的配置，帐号，密码，策略名##############
trader_port = 8080
trader_server_ip = u"101.201.196.115"
# trader_server_ip = u"127.0.0.1"
quote_port = 8180
#quote_server_ip = u"127.0.0.1"
quote_server_ip = u"211.152.51.189"
# account = "tangtao"
password_nodes = "96e79218965eb72c92a549dd5a330112"
account = "zcq"
# password = "96e79218965eb72c92a549dd5a330112"
password = "111111"
# password="96e79218965ea549dd5a330112"
strategy_id = "tangtao-test-strategy"

trader_conn_info = StrategyConnectionInfo(
    ip=trader_server_ip,
    port=trader_port,
    account_id=account,
    password=password,
    strategy_id=strategy_id
)
trader_config = StrategyConfiguration(
    trader_server_ip, trader_port, strategy_connection_info=trader_conn_info,
    usage_type=AgentType.STRATEGY_TRADE.get_value())
quote_conn_info = StrategyConnectionInfo(
    ip=quote_server_ip,
    port=quote_port,
    account_id=account,
    password=password_nodes,
    strategy_id=strategy_id
)
quote_config = StrategyConfiguration(
    quote_server_ip, quote_port, strategy_connection_info=quote_conn_info,
    usage_type=AgentType.STRATEGY_QUOTE.get_value())




# encoding: utf-8

__author__ = u'Yonka'


class ConnectionBrokenException(RuntimeError):
    pass


class AuthFailedException(RuntimeError):
    pass


class RequestTimeoutException(RuntimeError):
    pass


class InvalidRequestRuntimeException(RuntimeError):
    pass



# encoding: utf-8

from __future__ import absolute_import
import unittest
from sd.utils.enum import CommonNameValueEnum, EnumMetaCls, can_not_init

__author__ = u'yonka'


class AgentType(CommonNameValueEnum):
    __metaclass__ = EnumMetaCls

    def __str__(self):
        return u"AgentType(%d)" % self.v


AgentType.STRATEGY_TRADE = AgentType(1)
AgentType.STRATEGY_QUOTE = AgentType(2)

AgentType.__init__ = can_not_init


class TestAgentType(unittest.TestCase):
    def test_init(self):
        print AgentType.STRATEGY_TRADE
        res = False
        try:
            AgentType(1)
        except AssertionError:
            res = True
        assert res



# encoding: utf-8
from __future__ import absolute_import
import json
import logging

from sd.utils import protobuf_to_dict
from sd.strategy.bean import protoquote

__author__ = u'Yonka'


class BaseBean(object):
    def to_json(self):
        return json.dumps(self.to_dict())

    def to_dict(self):
        def _get_v_from_type(t, v):
            v_to_dict = getattr(t, u"to_dict", None)
            if v_to_dict is None:
                return unicode(v)
            else:
                return t.to_dict(v)

        d = dict(self.__dict__)
        types = getattr(self.__class__, u"_types", None)
        if not types:
            return d
        for k, t in types.items():
            v = d.get(k)
            if v is None:
                continue
            if issubclass(t, list):
                params = getattr(t, u"__parameters__", None)
                if params is None:
                    continue
                v = [_get_v_from_type(params[0], vi) for vi in v]
            else:
                v = _get_v_from_type(t, v)
            d[k] = v
        return d

    def from_json(self, s):
        u"""use json content to fulfill current instance
        @:param s json str
        @:return current instance
        """
        d = json.loads(s)
        return self.from_dict(d)

    def from_dict(self, d):
        def _get_v_from_constructor(c, v):
            params = getattr(c, u"__parameters__", None)
            if params:
                if issubclass(c, list):
                    return [_get_v_from_constructor(params[0], vi) for vi in v]
            handler = None
            if hasattr(c, u"from_value"):
                handler = getattr(c, u"from_value")
            elif hasattr(c, u"from_dict"):
                handler = getattr(c, u"from_dict")
            if handler is None:
                return c(v)
            else:
                res = c()
                handler(res, v)
                return res

        if not d:
            return

        types = getattr(self.__class__, u"_types", None)
        for k, v in d.items():
            if k in self.__dict__:
                if types and k in types:
                    constructor = types[k]
                    v = _get_v_from_constructor(constructor, v)
                self.__dict__[k] = v  # enum呢？ 非基本类型呢？，自行指定类型信息
        return self


def proto_to_bean(s, proto_t, bean_t):
    proto_t_obj = proto_t()
    proto_t_obj.ParseFromString(s)
    return bean_t().from_dict(protobuf_to_dict.protobuf_to_dict(proto_t_obj))


def json_load_bean_list(s, t, l=None):
    """
    @:param t type, should be subclass of BaseBean
    """
    j = json.loads(s)
    if j is None:
        return None
    elif not isinstance(j, list):
        logging.error("s: (%s) is not list", s)
        raise ValueError("not list")
    if l is None:
        l = []
    for ji in j:
        l.append(t().from_dict(ji))
    return l


def history_data_proto_2_bean_list(s, proto_t, t, l=None):
    """
    @:param t type, should be subclass of BaseBean
    """
    j = protoquote.HistoryDataBean()
    j.ParseFromString(s)
    if l is None:
        l = []
    for ji in j.historyData:
        l.append(proto_to_bean(ji, proto_t, t))
    return l



# encoding: utf-8

from __future__ import absolute_import

from sd.utils.bean import BaseBean



class KlineBean(BaseBean):
    def __init__(
            self,
            instrumentID=None,
            tradingDay=None,
            updateTime=None,
            openPrice=None,
            highPrice=None,
            lowPrice=None,
            closePrice=None,
            volume=None,
            turover=None,
            matchItems=None,
            interest=None,
            settlePrice=None
    ):
        self.instrumentID = instrumentID
        self.tradingDay = tradingDay
        self.updateTime = updateTime
        self.openPrice = openPrice
        self.highPrice = highPrice
        self.lowPrice = lowPrice
        self.closePrice = closePrice
        self.volume = volume
        self.turover = turover
        self.matchItems = matchItems
        self.interest = interest
        self.settlePrice = settlePrice


class TickBean(BaseBean):
    def __init__(
            self,
            instrumentID="",  # 合约ID SH.rb1605
            tradingDay=0,  # 交易日
            lastPrice=0,  # 最新价
            preClosePrice=0,  # 昨收盘价
            openPrice=0,  # 开盘价
            highestPrice=0,  # 最高价
            lowestPrice=0,  # 最低价
            volume=0,  # 成交总量
            turnover=0,  # 成交总金额
            openInterest=0,  # 持仓总量
            upperLimitPrice=0,  # 涨停价
            lowerLimitPrice=0,  # 跌停价
            updateTime=0,  # 交易所时间

            askPrice=[],  # 买价
            askVolume=[],
            bidPrice=[],
            bidVolume=[],

            # 期货
            settlementPrice=None,  # 今结算,期货

            # 期权属性
            preDelta=None,  # 昨虚实度
            currDela=None,  # 今虚实度
            contractID=None,  # 期权合约代码
            underlyingSecurityID=None,  # 标的证券代码
            callOrPut=None,  # 认购认沽
            exerciseDate=None,  # 行权日
            # 股票属性
            numTrades=None,
            totalBidVolume=None,
            totalAskVolume=None,
            weightedAvgBidPrice=None,
    ):
            self.instrumentID = instrumentID  # 合约ID SH.rb1605
            self.tradingDay = tradingDay  # 交易日
            self.lastPrice = lastPrice  # 最新价
            self.preClosePrice = preClosePrice  # 昨收盘价
            self.openPrice = openPrice  # 开盘价
            self.highestPrice = highestPrice  # 最高价
            self.lowestPrice = lowestPrice  # 最低价
            self.volume = volume  # 成交总量
            self.turnover = turnover  # 成交总金额
            self.openInterest = openInterest  # 持仓总量
            self.upperLimitPrice = upperLimitPrice  # 涨停价
            self.lowerLimitPrice = lowerLimitPrice  # 跌停价
            self.updateTime = updateTime  # 交易所时间

            self.askPrice = askPrice  # 买价
            self.askVolume = askVolume
            self.bidPrice = bidPrice
            self.bidVolume = bidVolume

            # 期货
            self.settlementPrice = settlementPrice  # 期货

            # 期权属性
            self.preDelta = preDelta  # 昨虚实度
            self.currDela = currDela  # 今虚实度
            self.contractID = contractID  # 期权合约代码
            self.underlyingSecurityID = underlyingSecurityID  # 标的证券代码
            self.callOrPut = callOrPut  # 认购认沽
            self.exerciseDate = exerciseDate  # 行权日
            # 股票属性
            self.numTrades = numTrades
            self.totalBidVolume = totalBidVolume
            self.totalAskVolume = totalAskVolume
            self.weightedAvgBidPrice = weightedAvgBidPrice

class FutureMarketBean(BaseBean):
    def __init__(
            self,
            instrumentID=None,  # 合约ID SH.rb1605
            actionDay=None,  # 业务发生日(自然日)
            tradingDay=None,  # 交易日
            lastPrice=0,  # 最新价
            preSettlementPrice=0,  # 昨结算
            preClosePrice=0,  # 昨收盘价
            preOpenInterest=0,  # 昨持仓
            openPrice=0,  # 开盘价
            highestPrice=0,  # 最高价
            lowestPrice=0,  # 最低价
            volume=0,  # 成交总量
            turnover=0,  # 成交总金额
            openInterest=0,  # 持仓总量
            closePrice=0,  # 今收盘
            settlementPrice=0,  # 今结算
            upperLimitPrice=0,  # 涨停价
            lowerLimitPrice=0,  # 跌停价
            preDelta=0,  # 昨虚实度
            currDela=0,  # 今虚实度
            updateTime=None,  # 交易所时间
            averagePrice=0,  # 均价

            askPrice=[],  # 买价
            askVolume=[],
            bidPrice=[],
            bidVolume=[],

            # 期权属性
            contractID=None,  # 期权合约代码
            underlyingSecurityID=None,  # 标的证券代码
            callOrPut=None,  # 认购认沽
            exerciseDate=None,  # 行权日
    ):
        self.instrumentID = instrumentID
        self.actionDay = actionDay
        self.tradingDay = tradingDay
        self.lastPrice = lastPrice
        self.preSettlementPrice = preSettlementPrice
        self.preClosePrice = preClosePrice
        self.preOpenInterest = preOpenInterest
        self.openPrice = openPrice
        self.highestPrice = highestPrice
        self.lowestPrice = lowestPrice
        self.volume = volume
        self.turnover = turnover
        self.openInterest = openInterest
        self.closePrice = closePrice
        self.settlementPrice = settlementPrice
        self.upperLimitPrice = upperLimitPrice
        self.lowerLimitPrice = lowerLimitPrice
        self.preDelta = preDelta
        self.currDela = currDela
        self.updateTime = updateTime
        self.averagePrice = averagePrice
        self.askPrice = askPrice
        self.askVolume = askVolume
        self.bidPrice = bidPrice
        self.bidVolume = bidVolume
        self.contractID = contractID
        self.underlyingSecurityID = underlyingSecurityID
        self.callOrPut = callOrPut
        self.exerciseDate = exerciseDate


class StockMarketBean(BaseBean):
    def __init__(
            self,
            instrumentID=None,  # 合约代码 SH.00000001
            prefix=None,  # 证券信息前缀
            actionDay=None,  # 业务发生日(自然日)
            tradingDay=None,  # 交易日
            updateTime=None,  # 交易所时间
            lastPrice=0,  # 最新价
            preClosePrice=0,  # 前收
            openPrice=0,  # 开盘价
            highPrice=0,  # 最高价
            lowPrice=0,  # 最低价
            numTrades=0,  # 成交笔数
            volume=0,  # 成交总量
            turnover=0,  # 成交总金额
            totalBidVolume=0,  # 委托买入总量
            totalAskVolume=0,  # 委托卖出总量
            weightedAvgBidPrice=0,  # 加权平均委买价格
            weightedAvgAskPrice=0,  # 加权平均委卖价格
            IOPV=0,  # IOPV净值估值
            yieldToMaturity=0,  # 到期收益率
            highLimitedPrice=0,  # 涨停价
            lowLimitedPrice=0,  # 跌停价
            syl1=0,  # 市盈率1
            syl2=0,  # 市盈率2
            SD2=0,  # 升跌2（对比上一笔）
            askPrice=[],
            askVolume=[],
            bidPrice=[],
            bidVolume=[]
    ):
        self.instrumentID = instrumentID
        self.prefix = prefix
        self.actionDay = actionDay
        self.tradingDay = tradingDay
        self.updateTime = updateTime
        self.lastPrice = lastPrice
        self.preClosePrice = preClosePrice
        self.openPrice = openPrice
        self.highPrice = highPrice
        self.lowPrice = lowPrice
        self.numTrades = numTrades
        self.volume = volume
        self.turnover = turnover
        self.totalBidVolume = totalBidVolume
        self.totalAskVolume = totalAskVolume
        self.weightedAvgBidPrice = weightedAvgBidPrice
        self.weightedAvgAskPrice = weightedAvgAskPrice
        self.IOPV = IOPV
        self.yieldToMaturity = yieldToMaturity
        self.highLimitedPrice = highLimitedPrice
        self.lowLimitedPrice = lowLimitedPrice
        self.syl1 = syl1
        self.syl2 = syl2
        self.SD2 = SD2
        self.askPrice = askPrice
        self.askVolume = askVolume
        self.bidPrice = bidPrice
        self.bidVolume = bidVolume



# encoding: utf-8
from __future__ import absolute_import
import time

__author__ = u'yonka'


def schedule_task(fn, initial_delay=0, period=0, repeat=1):
    u"""
    @:param fn should handle exception itself
    @:param repeat <= 0 means infinite repeat
    """

    def _fn(*args, **kwargs):
        cnt = 0
        if initial_delay > 0:
            time.sleep(initial_delay)
        while repeat <= 0 or repeat > cnt:
            cnt += 1
            fn(*args, **kwargs)
            if period > 0:
                time.sleep(period)

    return _fn


def timestamp():
    return int(time.time() * 1000)



# encoding: utf-8

from __future__ import absolute_import
from abc import ABCMeta, abstractmethod

__author__ = u'Yonka'


class AbstractFramer(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def frame_msg(self, message):
        pass

    @abstractmethod
    def next_msg(self):
        pass



# encoding: utf-8
from __future__ import absolute_import
import struct
import unittest

from sd.communication.server.protocol import function_code
from sd.communication.server.protocol.interface import AbstractProtocolMessage

__author__ = u'Yonka'


class ProtocolMessage(AbstractProtocolMessage):
    def __init__(
            self,
            fc=0,
            req_id=0,
            data_type=0,
            body=None
    ):
        self.fc = fc
        self.reqId = req_id
        self.dataType = data_type
        self.byteBody = body
        if self.byteBody is not None:
            try:
                self.body = self.byteBody.decode("utf-8")
            except Exception, e:
                self.body = "not utf-8"




    # use struct pack instead of convert_int_to_byte_array and etc, according to java code, we use little endian here
    def to_stream_bytes(self, endian=""):
        if endian is None:
            endian = ""
        res = ""
        if self.fc >= 0 and self.reqId >= 0 and self.dataType >= 0:
            body_len, body_bs = 0, None
            if self.body is not None:
                body_bs = self.body.encode("utf-8")
                body_len = len(body_bs)
            if body_len == 0:
                res = struct.pack(endian + "lqll", self.fc, self.reqId, self.dataType, 0)
            else:
                res = struct.pack(endian + "lqll%ds" % body_len, self.fc, self.reqId, self.dataType, body_len, body_bs)
        return res

    def is_hb_message(self):
        return self.fc == function_code.HEARTBEAT

    def is_ack_message(self):
        return self.fc == function_code.ACK

    def __str__(self):
        return u"ProtocolMessage(fc: %d, reqId: %d, dataType: %d, body: %s)" % (
            self.fc, self.reqId, self.dataType, self.body)


class TestProtocolMessage(unittest.TestCase):
    def test_to_stream_bytes(self):
        msg1 = ProtocolMessage(1, 1, 1, "123")
        msg2 = ProtocolMessage(1, 1, 1, "")
        msg3 = ProtocolMessage(-1, 1, 1, "123213")
        try:
            assert msg1.to_stream_bytes() == b"\x01\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00" \
                                             "\x03\x00\x00\x00123"
        except AssertionError, e:
            print u"msg1.to_stream_bytes(): %s" % msg1.to_stream_bytes()
            raise e
        try:
            assert msg2.to_stream_bytes() == "\x01\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00" \
                                             "\x00\x00\x00\x00"
        except AssertionError, e:
            print u"msg2.to_stream_bytes(): %s" % msg2.to_stream_bytes()
            raise e
        try:
            assert msg3.to_stream_bytes() == ""
        except AssertionError, e:
            print u"msg3.to_stream_bytes(): %s" % msg3.to_stream_bytes()
            raise e



# encoding: utf-8
from __future__ import absolute_import
from sd.communication.server.protocol import function_code
from sd.communication.server.protocol.protocol import ProtocolMessage

__author__ = u'Yonka'

HEADER_SIZE = 20
CLIENT_ID_SIZE = 4
REQID_SIZE = 4
PC_SIZE = 4
LEN_SIZE = 4
MAX_SIZE = 15000000

hb = ProtocolMessage(function_code.HEARTBEAT, 0, function_code.STRATEGY, u"")
hb_ack = ProtocolMessage(function_code.ACK, 0, function_code.STRATEGY, u"")


def new_strategy_hb_msg():
    return hb


def new_strategy_hb_ack_msg():
    return hb_ack
